import 'package:flutter/material.dart';
import 'package:flutter_svg/svg.dart';
import 'package:syncrow_web/utils/color_manager.dart';
import 'package:syncrow_web/utils/constants/assets.dart';
import 'package:syncrow_web/utils/style.dart';

class DynamicTableScreen extends StatefulWidget {
  final List<String> titles;
  final List<List<Widget>> rows;
  final void Function(int columnIndex)? onFilter;

  DynamicTableScreen(
      {required this.titles, required this.rows, required this.onFilter});

  @override
  _DynamicTableScreenState createState() => _DynamicTableScreenState();
}

class _DynamicTableScreenState extends State<DynamicTableScreen>
    with WidgetsBindingObserver {
  late List<double> columnWidths;
  late double totalWidth;

  @override
  void initState() {
    super.initState();
    columnWidths = List<double>.filled(widget.titles.length, 150.0);
    totalWidth = columnWidths.reduce((a, b) => a + b);
    WidgetsBinding.instance.addObserver(this);
  }

  @override
  void dispose() {
    WidgetsBinding.instance.removeObserver(this);
    super.dispose();
  }

  @override
  void didChangeMetrics() {
    super.didChangeMetrics();
    final newScreenWidth = MediaQuery.of(context).size.width;
    setState(() {
      columnWidths = List<double>.generate(widget.titles.length, (index) {
        if (index == 1) {
          return newScreenWidth *
              0.12; // 20% of screen width for the second column
        } else if (index == 9) {
          return newScreenWidth *
              0.1; // 25% of screen width for the tenth column
        }
        return newScreenWidth *
            0.09; // Default to 10% of screen width for other columns
      });
    });
  }

  @override
  Widget build(BuildContext context) {
    final screenWidth = MediaQuery.of(context).size.width;
    if (columnWidths.every((width) => width == screenWidth * 7)) {
      columnWidths = List<double>.generate(widget.titles.length, (index) {
        if (index == 1) {
          return screenWidth * 0.11;
        } else if (index == 9) {
          return screenWidth * 0.1;
        }
        return screenWidth * 0.09;
      });
      setState(() {});
    }
    return SingleChildScrollView(
      clipBehavior: Clip.none,
      scrollDirection: Axis.horizontal,
      child: Container(
        decoration: containerDecoration.copyWith(
            color: ColorsManager.whiteColors,
            borderRadius: const BorderRadius.all(Radius.circular(20))),
        child: FittedBox(
          child: Column(
            children: [
              Container(
                width: totalWidth,
                decoration: containerDecoration.copyWith(
                    color: ColorsManager.circleRolesBackground,
                    borderRadius: const BorderRadius.only(
                        topLeft: Radius.circular(15),
                        topRight: Radius.circular(15))),
                child: Row(
                  children: List.generate(widget.titles.length, (index) {
                    return Row(
                      mainAxisAlignment: MainAxisAlignment.start,
                      children: [
                        FittedBox(
                          child: Container(
                            padding: const EdgeInsets.only(left: 5, right: 5),
                            width: columnWidths[index],
                            child: Row(
                              mainAxisAlignment: MainAxisAlignment.spaceBetween,
                              crossAxisAlignment: CrossAxisAlignment.center,
                              children: [
                                SizedBox(
                                  child: Text(
                                    widget.titles[index],
                                    maxLines: 2,
                                    style: const TextStyle(
                                      overflow: TextOverflow.ellipsis,
                                      fontWeight: FontWeight.w400,
                                      fontSize: 13,
                                      color: ColorsManager.grayColor,
                                    ),
                                  ),
                                ),
                                if (index != 1 &&
                                    index != 9 &&
                                    index != 8 &&
                                    index != 5)
                                  FittedBox(
                                    child: IconButton(
                                      icon: SvgPicture.asset(
                                        Assets.filterTableIcon,
                                        fit: BoxFit.none,
                                      ),
                                      onPressed: () {
                                        if (widget.onFilter != null) {
                                          widget.onFilter!(index);
                                        }
                                      },
                                    ),
                                  )
                              ],
                            ),
                          ),
                        ),
                        GestureDetector(
                          onHorizontalDragUpdate: (details) {
                            setState(() {
                              columnWidths[index] =
                                  (columnWidths[index] + details.delta.dx)
                                      .clamp(150.0, 300.0);
                              totalWidth = columnWidths.reduce((a, b) => a + b);
                            });
                          },
                          child: MouseRegion(
                            cursor: SystemMouseCursors.resizeColumn,
                            child: Container(
                              color: Colors.green,
                              child: Container(
                                color: ColorsManager.boxDivider,
                                width: 1,
                                height: 50,
                              ),
                            ),
                          ),
                        ),
                      ],
                    );
                  }),
                ),
              ),
              widget.rows.isEmpty
                  ? SizedBox(
                      height: MediaQuery.of(context).size.height / 2,
                      child: Column(
                        crossAxisAlignment: CrossAxisAlignment.center,
                        mainAxisAlignment: MainAxisAlignment.center,
                        children: [
                          Column(
                            children: [
                              SvgPicture.asset(Assets.emptyTable),
                              const SizedBox(
                                height: 15,
                              ),
                              const Text(
                                'No Users',
                                style: TextStyle(
                                    color: ColorsManager.lightGrayColor,
                                    fontSize: 16,
                                    fontWeight: FontWeight.w700),
                              )
                            ],
                          ),
                        ],
                      ),
                    )
                  : Center(
                      child: Container(
                        width: totalWidth,
                        decoration: containerDecoration.copyWith(
                            color: ColorsManager.whiteColors,
                            borderRadius: const BorderRadius.only(
                                bottomLeft: Radius.circular(15),
                                bottomRight: Radius.circular(15))),
                        child: ListView.builder(
                          physics: const NeverScrollableScrollPhysics(),
                          shrinkWrap: true,
                          itemCount: widget.rows.length,
                          itemBuilder: (context, rowIndex) {
                            if (columnWidths.every((width) => width == 120.0)) {
                              columnWidths = List<double>.generate(
                                  widget.titles.length, (index) {
                                if (index == 1) {
                                  return screenWidth * 0.11;
                                } else if (index == 9) {
                                  return screenWidth * 0.2;
                                }
                                return screenWidth * 0.11;
                              });
                              setState(() {});
                            }
                            final row = widget.rows[rowIndex];
                            return Column(
                              children: [
                                Container(
                                  child: Padding(
                                    padding: const EdgeInsets.only(
                                        left: 5, top: 10, right: 5, bottom: 10),
                                    child: Row(
                                      children:
                                          List.generate(row.length, (index) {
                                        return SizedBox(
                                          width: columnWidths[index],
                                          child: SizedBox(
                                            child: Padding(
                                              padding: const EdgeInsets.only(
                                                  left: 15, right: 10),
                                              child: row[index],
                                            ),
                                          ),
                                        );
                                      }),
                                    ),
                                  ),
                                ),
                                if (rowIndex < widget.rows.length - 1)
                                  Row(
                                    children: List.generate(
                                        widget.titles.length, (index) {
                                      return SizedBox(
                                        width: columnWidths[index],
                                        child: const Divider(
                                          color: ColorsManager.boxDivider,
                                          thickness: 1,
                                          height: 1,
                                        ),
                                      );
                                    }),
                                  ),
                              ],
                            );
                          },
                        ),
                      ),
                    ),
            ],
          ),
        ),
      ),
    );
  }
}
