// Flutter imports
import 'package:flutter/material.dart';
import 'package:flutter_bloc/flutter_bloc.dart';

// Syncrow project imports
import 'package:syncrow_web/pages/common/buttons/add_space_button.dart';
import 'package:syncrow_web/pages/common/buttons/cancel_button.dart';
import 'package:syncrow_web/pages/common/buttons/default_button.dart';
import 'package:syncrow_web/pages/spaces_management/all_spaces/bloc/space_management_bloc.dart';
import 'package:syncrow_web/pages/spaces_management/all_spaces/bloc/space_management_event.dart';
import 'package:syncrow_web/pages/spaces_management/all_spaces/model/product_model.dart';
import 'package:syncrow_web/pages/spaces_management/all_spaces/model/selected_product_model.dart';
import 'package:syncrow_web/pages/spaces_management/all_spaces/model/space_model.dart';
import 'package:syncrow_web/pages/spaces_management/all_spaces/model/community_model.dart';
import 'package:syncrow_web/pages/spaces_management/all_spaces/model/connection_model.dart';
import 'package:syncrow_web/pages/spaces_management/all_spaces/model/subspace_model.dart';
import 'package:syncrow_web/pages/spaces_management/all_spaces/model/tag.dart';
import 'package:syncrow_web/pages/spaces_management/all_spaces/widgets/blank_community_widget.dart';
import 'package:syncrow_web/pages/spaces_management/all_spaces/widgets/community_structure_header_widget.dart';
import 'package:syncrow_web/pages/spaces_management/all_spaces/widgets/dialogs/create_space_dialog.dart';
import 'package:syncrow_web/pages/spaces_management/all_spaces/widgets/curved_line_painter.dart';
import 'package:syncrow_web/pages/spaces_management/all_spaces/widgets/dialogs/duplicate_process_dialog.dart';
import 'package:syncrow_web/pages/spaces_management/all_spaces/widgets/space_card_widget.dart';
import 'package:syncrow_web/pages/spaces_management/all_spaces/widgets/space_container_widget.dart';
import 'package:syncrow_web/pages/spaces_management/helper/connection_helper.dart';
import 'package:syncrow_web/pages/spaces_management/helper/space_helper.dart';
import 'package:syncrow_web/pages/spaces_management/helper/tag_helper.dart';
import 'package:syncrow_web/pages/spaces_management/space_model/models/space_template_model.dart';
import 'package:syncrow_web/utils/color_manager.dart';

class CommunityStructureArea extends StatefulWidget {
  final CommunityModel? selectedCommunity;
  SpaceModel? selectedSpace;
  final List<ProductModel>? products;
  final ValueChanged<SpaceModel?>? onSpaceSelected;
  final List<CommunityModel> communities;
  final List<SpaceModel> spaces;
  final List<SpaceTemplateModel>? spaceModels;

  CommunityStructureArea({
    this.selectedCommunity,
    this.selectedSpace,
    required this.communities,
    this.products,
    required this.spaces,
    this.onSpaceSelected,
    this.spaceModels,
  });

  @override
  _CommunityStructureAreaState createState() => _CommunityStructureAreaState();
}

class _CommunityStructureAreaState extends State<CommunityStructureArea> {
  double canvasWidth = 1000;
  double canvasHeight = 1000;
  List<SpaceModel> spaces = [];
  List<Connection> connections = [];
  late TextEditingController _nameController;
  bool isEditingName = false;
  late TransformationController _transformationController;

  @override
  void initState() {
    super.initState();
    spaces = widget.spaces.isNotEmpty ? flattenSpaces(widget.spaces) : [];
    connections =
        widget.spaces.isNotEmpty ? createConnections(widget.spaces) : [];
    _adjustCanvasSizeForSpaces();
    _nameController = TextEditingController(
      text: widget.selectedCommunity?.name ?? '',
    );
    _transformationController = TransformationController();
    if (widget.selectedSpace != null) {
      WidgetsBinding.instance.addPostFrameCallback((_) {
        _moveToSpace(widget.selectedSpace!);
      });
    }
  }

  @override
  void dispose() {
    _nameController.dispose();
    _transformationController.dispose();
    super.dispose();
  }

  @override
  void didUpdateWidget(covariant CommunityStructureArea oldWidget) {
    super.didUpdateWidget(oldWidget);

    if (oldWidget.spaces != widget.spaces) {
      setState(() {
        spaces = widget.spaces.isNotEmpty ? flattenSpaces(widget.spaces) : [];
        connections =
            widget.spaces.isNotEmpty ? createConnections(widget.spaces) : [];
        _adjustCanvasSizeForSpaces();
      });
    }

    if (widget.selectedSpace != oldWidget.selectedSpace &&
        widget.selectedSpace != null) {
      WidgetsBinding.instance.addPostFrameCallback((_) {
        _moveToSpace(widget.selectedSpace!);
      });
    }
  }

  @override
  Widget build(BuildContext context) {
    if (widget.selectedCommunity == null) {
      return BlankCommunityWidget(
        communities: widget.communities,
      );
    }

    Size screenSize = MediaQuery.of(context).size;
    return Expanded(
        child: GestureDetector(
      onTap: () {
        _deselectSpace(context);
      },
      child: Container(
        decoration: const BoxDecoration(
          border: Border(
            left: BorderSide(color: ColorsManager.whiteColors, width: 1.0),
          ),
        ),
        child: Column(
          crossAxisAlignment: CrossAxisAlignment.start,
          children: [
            CommunityStructureHeader(
              communities: widget.communities,
              communityName: widget.selectedCommunity?.name,
              community: widget.selectedCommunity,
              isSave: SpaceHelper.isSave(spaces),
              isEditingName: isEditingName,
              nameController: _nameController,
              onSave: _saveSpaces,
              selectedSpace: widget.selectedSpace,
              onDelete: _onDelete,
              onDuplicate: () => {_onDuplicate(context)},
              onEdit: () => {_showEditSpaceDialog()},
              onEditName: () {
                setState(() {
                  isEditingName = !isEditingName;
                  if (isEditingName) {
                    _nameController.text = widget.selectedCommunity?.name ?? '';
                  }
                });
              },
              onNameSubmitted: (value) {
                context.read<SpaceManagementBloc>().add(
                      UpdateCommunityEvent(
                        communityUuid: widget.selectedCommunity!.uuid,
                        name: value,
                      ),
                    );
                setState(() {
                  widget.selectedCommunity?.name = value;
                  isEditingName = false;
                });
              },
            ),
            Flexible(
                child: Stack(
              children: [
                InteractiveViewer(
                  transformationController: _transformationController,
                  boundaryMargin: EdgeInsets.all(500),
                  minScale: 0.5,
                  maxScale: 3.0,
                  constrained: false,
                  child: Container(
                    width: canvasWidth,
                    height: canvasHeight,
                    child: Stack(
                      children: [
                        for (var connection in connections)
                          Opacity(
                            opacity: ConnectionHelper.isHighlightedConnection(
                                    connection, widget.selectedSpace)
                                ? 1.0
                                : 0.3, // Adjust opacity
                            child: CustomPaint(
                                painter: CurvedLinePainter([connection])),
                          ),
                        for (var entry in spaces.asMap().entries)
                          if (entry.value.status != SpaceStatus.deleted &&
                              entry.value.status != SpaceStatus.parentDeleted)
                            Positioned(
                              left: entry.value.position.dx,
                              top: entry.value.position.dy,
                              child: SpaceCardWidget(
                                index: entry.key,
                                onButtonTap: (int index, Offset newPosition,
                                    String direction) {
                                  _showCreateSpaceDialog(
                                    screenSize,
                                    position:
                                        spaces[index].position + newPosition,
                                    parentIndex: index,
                                    direction: direction,
                                  );
                                },
                                position: entry.value.position,
                                isHovered: entry.value.isHovered,
                                screenSize: screenSize,
                                onHoverChanged: _handleHoverChanged,
                                onPositionChanged: (newPosition) {
                                  _updateNodePosition(entry.value, newPosition);
                                },
                                buildSpaceContainer: (int index) {
                                  final bool isHighlighted =
                                      SpaceHelper.isHighlightedSpace(
                                          spaces[index], widget.selectedSpace);

                                  return Opacity(
                                      opacity: isHighlighted ? 1.0 : 0.3,
                                      child: SpaceContainerWidget(
                                        index: index,
                                        onTap: () {
                                          _selectSpace(context, spaces[index]);
                                        },
                                        icon: spaces[index].icon ?? '',
                                        name: spaces[index].name,
                                      ));
                                },
                              ),
                            ),
                      ],
                    ),
                  ),
                ),
                if (spaces.isEmpty)
                  Center(
                    child: AddSpaceButton(
                      onTap: () {
                        _showCreateSpaceDialog(screenSize,
                            canvasHeight: canvasHeight,
                            canvasWidth: canvasWidth);
                      },
                    ),
                  ),
              ],
            )),
          ],
        ),
      ),
    ));
  }

  void _updateNodePosition(SpaceModel node, Offset newPosition) {
    setState(() {
      node.position = newPosition;
      if (node.status != SpaceStatus.newSpace) {
        node.status = SpaceStatus.modified; // Mark as modified
      }
      if (node.position.dx >= canvasWidth - 200) {
        canvasWidth += 200;
      }
      if (node.position.dy >= canvasHeight - 200) {
        canvasHeight += 200;
      }
      if (node.position.dx <= 200) {
        double shiftAmount = 200;
        canvasWidth += shiftAmount;
        for (var n in spaces) {
          n.position = Offset(n.position.dx + shiftAmount, n.position.dy);
        }
      }
      if (node.position.dy < 0) {
        node.position = Offset(node.position.dx, 0);
      }
    });
  }

  void _adjustCanvasSizeForSpaces() {
    for (var space in spaces) {
      if (space.position.dx >= canvasWidth - 200) {
        canvasWidth = space.position.dx + 200;
      }

      if (space.position.dy >= canvasHeight - 200) {
        canvasHeight = space.position.dy + 200;
      }
    }
  }

  void _showCreateSpaceDialog(Size screenSize,
      {Offset? position,
      int? parentIndex,
      String? direction,
      double? canvasWidth,
      double? canvasHeight}) {
    showDialog(
      context: context,
      builder: (BuildContext context) {
        return CreateSpaceDialog(
          products: widget.products,
          spaceModels: widget.spaceModels,
          allTags:
              TagHelper.getAllTagValues(widget.communities, widget.spaceModels),
          parentSpace: parentIndex != null ? spaces[parentIndex] : null,
          onCreateSpace: (String name,
              String icon,
              List<SelectedProduct> selectedProducts,
              SpaceTemplateModel? spaceModel,
              List<SubspaceModel>? subspaces,
              List<Tag>? tags) {
            setState(() {
              // Set the first space in the center or use passed position
              Offset centerPosition =
                  position ?? ConnectionHelper.getCenterPosition(screenSize);
              SpaceModel newSpace = SpaceModel(
                  name: name,
                  icon: icon,
                  position: centerPosition,
                  isPrivate: false,
                  children: [],
                  status: SpaceStatus.newSpace,
                  spaceModel: spaceModel,
                  subspaces: subspaces,
                  tags: tags);

              if (parentIndex != null && direction != null) {
                SpaceModel parentSpace = spaces[parentIndex];
                parentSpace.internalId = spaces[parentIndex].internalId;
                newSpace.parent = parentSpace;
                final newConnection = Connection(
                  startSpace: parentSpace,
                  endSpace: newSpace,
                  direction: direction,
                );
                connections.add(newConnection);
                newSpace.incomingConnection = newConnection;
                parentSpace.addOutgoingConnection(newConnection);
                parentSpace.children.add(newSpace);
              }
              spaces.add(newSpace);
              _updateNodePosition(newSpace, newSpace.position);
            });
          },
        );
      },
    );
  }

  void _showEditSpaceDialog() {
    if (widget.selectedSpace != null) {
      showDialog(
        context: context,
        builder: (BuildContext context) {
          return CreateSpaceDialog(
            products: widget.products,
            spaceModels: widget.spaceModels,
            name: widget.selectedSpace!.name,
            icon: widget.selectedSpace!.icon,
            parentSpace: SpaceHelper.findSpaceByInternalId(
                widget.selectedSpace?.parent?.internalId, spaces),
            editSpace: widget.selectedSpace,
            currentSpaceModel: widget.selectedSpace?.spaceModel,
            tags: widget.selectedSpace?.tags,
            subspaces: widget.selectedSpace?.subspaces,
            isEdit: true,
            allTags: TagHelper.getAllTagValues(
                widget.communities, widget.spaceModels),
            onCreateSpace: (String name,
                String icon,
                List<SelectedProduct> selectedProducts,
                SpaceTemplateModel? spaceModel,
                List<SubspaceModel>? subspaces,
                List<Tag>? tags) {
              setState(() {
                // Update the space's properties
                widget.selectedSpace!.name = name;
                widget.selectedSpace!.icon = icon;
                widget.selectedSpace!.spaceModel = spaceModel;
                widget.selectedSpace!.subspaces = subspaces;
                widget.selectedSpace!.tags = tags;

                if (widget.selectedSpace!.status != SpaceStatus.newSpace) {
                  widget.selectedSpace!.status =
                      SpaceStatus.modified; // Mark as modified
                }

                for (var space in spaces) {
                  if (space.internalId == widget.selectedSpace?.internalId) {
                    space.status = SpaceStatus.modified;
                    space.subspaces = subspaces;
                    space.tags = tags;
                    space.name = name;
                  }
                }
              });
            },
            key: Key(widget.selectedSpace!.name),
          );
        },
      );
    }
  }

  void _handleHoverChanged(int index, bool isHovered) {
    setState(() {
      spaces[index].isHovered = isHovered;
    });
  }

  List<SpaceModel> flattenSpaces(List<SpaceModel> spaces) {
    List<SpaceModel> result = [];

    void flatten(SpaceModel space) {
      if (space.status == SpaceStatus.deleted ||
          space.status == SpaceStatus.parentDeleted) {
        return;
      }
      result.add(space);
      for (var child in space.children) {
        flatten(child);
      }
    }

    for (var space in spaces) {
      flatten(space);
    }

    return result;
  }

  List<Connection> createConnections(List<SpaceModel> spaces) {
    List<Connection> connections = [];

    void addConnections(SpaceModel parent, String direction) {
      if (parent.status == SpaceStatus.deleted) return;

      for (var child in parent.children) {
        if (child.status == SpaceStatus.deleted) continue;

        connections.add(
          Connection(
            startSpace: parent,
            endSpace: child,
            direction: child.incomingConnection?.direction ?? "down",
          ),
        );

        // Recursively process the child's children
        addConnections(child, direction);
      }
    }

    for (var space in spaces) {
      addConnections(space, "down");
    }

    return connections;
  }

  void _saveSpaces() {
    if (widget.selectedCommunity == null) {
      debugPrint("No community selected for saving spaces.");
      return;
    }

    List<SpaceModel> spacesToSave = spaces.where((space) {
      return space.status == SpaceStatus.newSpace ||
          space.status == SpaceStatus.modified ||
          space.status == SpaceStatus.deleted;
    }).toList();

    if (spacesToSave.isEmpty) {
      return;
    }

    String communityUuid = widget.selectedCommunity!.uuid;

    context.read<SpaceManagementBloc>().add(SaveSpacesEvent(
          spaces: spacesToSave,
          communityUuid: communityUuid,
        ));
  }

  void _onDelete() {
    if (widget.selectedSpace != null) {
      setState(() {
        for (var space in spaces) {
          if (space.internalId == widget.selectedSpace?.internalId) {
            space.status = SpaceStatus.deleted;
            _markChildrenAsDeleted(space);
          }
        }

        _removeConnectionsForDeletedSpaces();
      });
    }
  }

  void _markChildrenAsDeleted(SpaceModel parent) {
    for (var child in parent.children) {
      child.status = SpaceStatus.parentDeleted;

      _markChildrenAsDeleted(child);
    }
  }

  void _removeConnectionsForDeletedSpaces() {
    connections.removeWhere((connection) {
      return connection.startSpace.status == SpaceStatus.deleted ||
          connection.endSpace.status == SpaceStatus.deleted ||
          connection.startSpace.status == SpaceStatus.parentDeleted ||
          connection.endSpace.status == SpaceStatus.parentDeleted;
    });
  }

  void _moveToSpace(SpaceModel space) {
    final double viewportWidth = MediaQuery.of(context).size.width;
    final double viewportHeight = MediaQuery.of(context).size.height;

    final double dx = -space.position.dx + (viewportWidth / 2) - 400;
    final double dy = -space.position.dy + (viewportHeight / 2) - 300;

    _transformationController.value = Matrix4.identity()
      ..translate(dx, dy)
      ..scale(1.2);
  }

  void _selectSpace(BuildContext context, SpaceModel space) {
    context.read<SpaceManagementBloc>().add(
          SelectSpaceEvent(
              selectedCommunity: widget.selectedCommunity,
              selectedSpace: space),
        );
  }

  void _deselectSpace(BuildContext context) {
    context.read<SpaceManagementBloc>().add(
          SelectSpaceEvent(
              selectedCommunity: widget.selectedCommunity, selectedSpace: null),
        );
  }

  void _onDuplicate(BuildContext parentContext) {
    final screenWidth = MediaQuery.of(context).size.width;

    if (widget.selectedSpace != null) {
      showDialog(
        context: context,
        builder: (BuildContext context) {
          return AlertDialog(
            backgroundColor: ColorsManager.whiteColors,
            title: Text(
              "Duplicate Space",
              textAlign: TextAlign.center,
              style: Theme.of(context)
                  .textTheme
                  .headlineLarge
                  ?.copyWith(color: ColorsManager.blackColor),
            ),
            content: SizedBox(
              width: screenWidth * 0.4,
              child: Column(
                mainAxisSize: MainAxisSize.min,
                children: [
                  Text("Are you sure you want to duplicate?",
                      textAlign: TextAlign.center,
                      style: Theme.of(context).textTheme.headlineSmall),
                  const SizedBox(height: 15),
                  Text("All the child spaces will be duplicated.",
                      textAlign: TextAlign.center,
                      style: Theme.of(context)
                          .textTheme
                          .bodyMedium
                          ?.copyWith(color: ColorsManager.lightGrayColor)),
                  const SizedBox(width: 15),
                ],
              ),
            ),
            actions: [
              Row(mainAxisAlignment: MainAxisAlignment.center, children: [
                SizedBox(
                  width: 200,
                  child: CancelButton(
                    onPressed: () {
                      Navigator.of(context).pop();
                    },
                    label: "Cancel",
                  ),
                ),
                const SizedBox(width: 10),
                SizedBox(
                  width: 200,
                  child: DefaultButton(
                    onPressed: () {
                      Navigator.of(context).pop();
                      showDialog(
                        context: context,
                        barrierDismissible: false,
                        builder: (BuildContext context) {
                          return DuplicateProcessDialog(
                            onDuplicate: () {
                              _duplicateSpace(widget.selectedSpace!);
                              _deselectSpace(parentContext);
                            },
                          );
                        },
                      );
                    },
                    backgroundColor: ColorsManager.secondaryColor,
                    borderRadius: 10,
                    foregroundColor: ColorsManager.whiteColors,
                    child: const Text('OK'),
                  ),
                ),
              ])
            ],
          );
        },
      );
    }
  }

  void _duplicateSpace(SpaceModel space) {
    final Map<SpaceModel, SpaceModel> originalToDuplicate = {};
    const double horizontalGap = 200.0;
    const double verticalGap = 100.0;

    SpaceModel duplicateRecursive(SpaceModel original, Offset parentPosition,
        SpaceModel? duplicatedParent) {
      Offset newPosition =
          Offset(parentPosition.dx + horizontalGap, original.position.dy);

      while (spaces.any((s) =>
          (s.position - newPosition).distance < horizontalGap &&
          s.status != SpaceStatus.deleted)) {
        newPosition += Offset(horizontalGap, 0);
      }

      final duplicatedName =
          SpaceHelper.generateUniqueSpaceName(original.name, spaces);

      final duplicated = SpaceModel(
        name: duplicatedName,
        icon: original.icon,
        position: newPosition,
        isPrivate: original.isPrivate,
        children: [],
        status: SpaceStatus.newSpace,
        parent: duplicatedParent,
        spaceModel: original.spaceModel,
        subspaces: original.subspaces,
        tags: original.tags,
      );

      originalToDuplicate[original] = duplicated;

      setState(() {
        spaces.add(duplicated);
        _updateNodePosition(duplicated, duplicated.position);

        if (duplicatedParent != null) {
          final newConnection = Connection(
            startSpace: duplicatedParent,
            endSpace: duplicated,
            direction: original.incomingConnection?.direction ?? 'down',
          );
          connections.add(newConnection);
          duplicated.incomingConnection = newConnection;
          duplicatedParent.addOutgoingConnection(newConnection);
        }

        if (original.parent != null && duplicatedParent == null) {
          final originalParent = original.parent!;
          final duplicatedParent =
              originalToDuplicate[originalParent] ?? originalParent;

          final parentConnection = Connection(
            startSpace: duplicatedParent,
            endSpace: duplicated,
            direction: original.incomingConnection?.direction ?? "down",
          );

          connections.add(parentConnection);
          duplicated.incomingConnection = parentConnection;
          duplicatedParent.addOutgoingConnection(parentConnection);
        }
      });

      final childrenWithDownDirection = original.children
          .where((child) =>
              child.incomingConnection?.direction == "down" &&
              child.status != SpaceStatus.deleted)
          .toList();

      Offset childStartPosition = childrenWithDownDirection.length == 1
          ? duplicated.position
          : newPosition + Offset(0, verticalGap);

      for (final child in original.children) {
        final isDownDirection =
            child.incomingConnection?.direction == "down" ?? false;

        if (isDownDirection && childrenWithDownDirection.length == 1) {
          childStartPosition = duplicated.position + Offset(0, verticalGap);
        } else if (!isDownDirection) {
          childStartPosition = duplicated.position + Offset(horizontalGap, 0);
        }

        final duplicatedChild =
            duplicateRecursive(child, childStartPosition, duplicated);
        duplicated.children.add(duplicatedChild);
        childStartPosition += Offset(0, verticalGap);
      }

      return duplicated;
    }

    if (space.parent == null) {
      duplicateRecursive(space, space.position, null);
    } else {
      final duplicatedParent =
          originalToDuplicate[space.parent!] ?? space.parent!;
      duplicateRecursive(space, space.position, duplicatedParent);
    }
  }
}
