import 'package:flutter_bloc/flutter_bloc.dart';
import 'package:syncrow_web/pages/spaces_management/all_spaces/model/tag.dart';
import 'package:syncrow_web/pages/spaces_management/assign_tag/bloc/assign_tag_event.dart';
import 'package:syncrow_web/pages/spaces_management/assign_tag/bloc/assign_tag_state.dart';

class AssignTagBloc extends Bloc<AssignTagEvent, AssignTagState> {
  AssignTagBloc() : super(AssignTagInitial()) {
    on<InitializeTags>((event, emit) {
      final initialTags = event.initialTags ?? [];

      final existingTagCounts = <String, int>{};
      for (var tag in initialTags) {
        if (tag.product != null) {
          existingTagCounts[tag.product!.uuid] =
              (existingTagCounts[tag.product!.uuid] ?? 0) + 1;
        }
      }

      final allTags = <Tag>[];

      for (var selectedProduct in event.addedProducts) {
        final existingCount = existingTagCounts[selectedProduct.productId] ?? 0;

        if (selectedProduct.count == 0 ||
            selectedProduct.count <= existingCount) {
          allTags.addAll(initialTags
              .where((tag) => tag.product?.uuid == selectedProduct.productId));
          continue;
        }

        final missingCount = selectedProduct.count - existingCount;

        allTags.addAll(initialTags
            .where((tag) => tag.product?.uuid == selectedProduct.productId));

        if (missingCount > 0) {
          allTags.addAll(List.generate(
            missingCount,
            (index) => Tag(
              tag: '',
              product: selectedProduct.product,
              location: 'Main Space',
            ),
          ));
        }
      }

      emit(AssignTagLoaded(
          tags: allTags,
          isSaveEnabled: _validateTags(allTags),
          errorMessage: ''));
    });

    on<UpdateTagEvent>((event, emit) {
      final currentState = state;

      if (currentState is AssignTagLoaded && currentState.tags.isNotEmpty) {
        final tags = List<Tag>.from(currentState.tags);
        tags[event.index].tag = event.tag;

        emit(AssignTagLoaded(
          tags: tags,
          isSaveEnabled: _validateTags(tags),
          errorMessage: _getValidationError(tags),
        ));
      }
    });

    on<UpdateLocation>((event, emit) {
      final currentState = state;

      if (currentState is AssignTagLoaded && currentState.tags.isNotEmpty) {
        final tags = List<Tag>.from(currentState.tags);

        // Use copyWith for immutability
        tags[event.index] =
            tags[event.index].copyWith(location: event.location);

        emit(AssignTagLoaded(
          tags: tags,
          isSaveEnabled: _validateTags(tags),
          errorMessage: _getValidationError(tags),
        ));
      }
    });

    on<ValidateTags>((event, emit) {
      final currentState = state;

      if (currentState is AssignTagLoaded && currentState.tags.isNotEmpty) {
        final tags = List<Tag>.from(currentState.tags);

        emit(AssignTagLoaded(
          tags: tags,
          isSaveEnabled: _validateTags(tags),
          errorMessage: _getValidationError(tags),
        ));
      }
    });

    on<DeleteTag>((event, emit) {
      final currentState = state;

      if (currentState is AssignTagLoaded && currentState.tags.isNotEmpty) {
        final updatedTags = List<Tag>.from(currentState.tags)
          ..remove(event.tagToDelete);

        emit(AssignTagLoaded(
          tags: updatedTags,
          isSaveEnabled: _validateTags(updatedTags),
          errorMessage: _getValidationError(updatedTags),
        ));
      } else {
        emit(const AssignTagLoaded(
            tags: [],
            isSaveEnabled: false,
            errorMessage: 'Failed to delete tag'));
      }
    });
  }

  bool _validateTags(List<Tag> tags) {
    final uniqueTags = tags.map((tag) => tag.tag?.trim() ?? '').toSet();
    final hasEmptyTag = tags.any((tag) => (tag.tag?.trim() ?? '').isEmpty);
    final isValid = uniqueTags.length == tags.length && !hasEmptyTag;
    return isValid;
  }

  String? _getValidationError(List<Tag> tags) {
    final hasEmptyTag = tags.any((tag) => (tag.tag?.trim() ?? '').isEmpty);
    if (hasEmptyTag) {
      return 'Tags cannot be empty.';
    }

    final duplicateTags = tags
        .map((tag) => tag.tag?.trim() ?? '')
        .fold<Map<String, int>>({}, (map, tag) {
          map[tag] = (map[tag] ?? 0) + 1;
          return map;
        })
        .entries
        .where((entry) => entry.value > 1)
        .map((entry) => entry.key)
        .toList();

    if (duplicateTags.isNotEmpty) {
      return 'Duplicate tags found: ${duplicateTags.join(', ')}';
    }

    return null;
  }
}
