import 'package:flutter_bloc/flutter_bloc.dart';
import 'package:syncrow_web/pages/spaces_management/assign_tag_models/bloc/assign_tag_model_event.dart';
import 'package:syncrow_web/pages/spaces_management/assign_tag_models/bloc/assign_tag_model_state.dart';
import 'package:syncrow_web/pages/spaces_management/space_model/models/tag_model.dart';

class AssignTagModelBloc
    extends Bloc<AssignTagModelEvent, AssignTagModelState> {
  AssignTagModelBloc() : super(AssignTagModelInitial()) {
    on<InitializeTagModels>((event, emit) {
      final initialTags = event.initialTags ?? [];

      final existingTagCounts = <String, int>{};
      for (var tag in initialTags) {
        if (tag.product != null) {
          existingTagCounts[tag.product!.uuid] =
              (existingTagCounts[tag.product!.uuid] ?? 0) + 1;
        }
      }

      final allTags = <TagModel>[];

      for (var selectedProduct in event.addedProducts) {
        final existingCount = existingTagCounts[selectedProduct.productId] ?? 0;

        if (selectedProduct.count == 0 ||
            selectedProduct.count <= existingCount) {
          allTags.addAll(initialTags
              .where((tag) => tag.product?.uuid == selectedProduct.productId));
          continue;
        }

        final missingCount = selectedProduct.count - existingCount;

        allTags.addAll(initialTags
            .where((tag) => tag.product?.uuid == selectedProduct.productId));

        if (missingCount > 0) {
          allTags.addAll(List.generate(
            missingCount,
            (index) => TagModel(
              tag: '',
              product: selectedProduct.product,
              location: 'Main Space',
            ),
          ));
        }
      }

      emit(AssignTagModelLoaded(
          tags: allTags,
          isSaveEnabled: _validateTags(allTags),
          errorMessage: ''));
    });

    on<UpdateTag>((event, emit) {
      final currentState = state;
      if (currentState is AssignTagModelLoaded &&
          currentState.tags.isNotEmpty) {
        final tags = List<TagModel>.from(currentState.tags);
        tags[event.index].tag = event.tag;
        emit(AssignTagModelLoaded(
          tags: tags,
          isSaveEnabled: _validateTags(tags),
          errorMessage: _getValidationError(tags),
        ));
      }
    });

    on<UpdateLocation>((event, emit) {
      final currentState = state;

      if (currentState is AssignTagModelLoaded &&
          currentState.tags.isNotEmpty) {
        final tags = List<TagModel>.from(currentState.tags);

        // Use copyWith for immutability
        tags[event.index] =
            tags[event.index].copyWith(location: event.location);

        emit(AssignTagModelLoaded(
          tags: tags,
          isSaveEnabled: _validateTags(tags),
        ));
      }
    });

    on<ValidateTagModels>((event, emit) {
      final currentState = state;

      if (currentState is AssignTagModelLoaded &&
          currentState.tags.isNotEmpty) {
        final tags = List<TagModel>.from(currentState.tags);

        emit(AssignTagModelLoaded(
          tags: tags,
          isSaveEnabled: _validateTags(tags),
          errorMessage: _getValidationError(tags),
        ));
      }
    });

    on<DeleteTagModel>((event, emit) {
      final currentState = state;

      if (currentState is AssignTagModelLoaded &&
          currentState.tags.isNotEmpty) {
        final updatedTags = List<TagModel>.from(currentState.tags)
          ..remove(event.tagToDelete);

        emit(AssignTagModelLoaded(
          tags: updatedTags,
          isSaveEnabled: _validateTags(updatedTags),
        ));
      } else {
        emit(const AssignTagModelLoaded(
          tags: [],
          isSaveEnabled: false,
        ));
      }
    });
  }

  bool _validateTags(List<TagModel> tags) {
    
    final uniqueTags = tags.map((tag) => tag.tag?.trim() ?? '').toSet();
    final hasEmptyTag = tags.any((tag) => (tag.tag?.trim() ?? '').isEmpty);
    final isValid = uniqueTags.length == tags.length && !hasEmptyTag;
    return isValid;
  }

  String? _getValidationError(List<TagModel> tags) {
    final hasEmptyTag = tags.any((tag) => (tag.tag?.trim() ?? '').isEmpty);
    if (hasEmptyTag) {
      return 'Tags cannot be empty.';
    }

    // Check for duplicate tags
    final duplicateTags = tags
        .map((tag) => tag.tag?.trim() ?? '')
        .fold<Map<String, int>>({}, (map, tag) {
          map[tag] = (map[tag] ?? 0) + 1;
          return map;
        })
        .entries
        .where((entry) => entry.value > 1)
        .map((entry) => entry.key)
        .toList();

    if (duplicateTags.isNotEmpty) {
      return 'Duplicate tags found: ${duplicateTags.join(', ')}';
    }

    return null;
  }
}
